/*
 * Copyright ETH 2014 - 2023 Zürich, Scientific IT Services
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ch.systemsx.cisd.openbis.generic.shared.api.v1.filter;

import org.testng.AssertJUnit;
import org.testng.annotations.Test;

import ch.systemsx.cisd.openbis.generic.shared.api.v1.dto.DataSet;
import ch.systemsx.cisd.openbis.generic.shared.api.v1.dto.DataSet.DataSetInitializer;
import ch.systemsx.cisd.openbis.generic.shared.api.v1.dto.EntityRegistrationDetails;
import ch.systemsx.cisd.openbis.generic.shared.api.v1.dto.EntityRegistrationDetails.EntityRegistrationDetailsInitializer;

/**
 * @author Franz-Josef Elmer
 */
public class TypeBasedDataSetFilterTest extends AssertJUnit
{

    @Test
    public void test()
    {
        TypeBasedDataSetFilter filter = new TypeBasedDataSetFilter("AB.*");

        assertPassResult(true, filter, "ABC");
        assertPassResult(false, filter, "aBC");
    }

    private void assertPassResult(boolean expected, TypeBasedDataSetFilter filter, String typeCode)
    {
        DataSetInitializer initializer = new DataSetInitializer();
        initializer.setCode("DS-1");
        initializer.setDataSetTypeCode(typeCode);
        EntityRegistrationDetailsInitializer detailsInitializer = new EntityRegistrationDetailsInitializer();
        initializer.setRegistrationDetails(new EntityRegistrationDetails(detailsInitializer));
        assertEquals(expected, filter.pass(new DataSet(initializer)));
    }

}
